from datetime import datetime, timedelta
import time
import pandas as pd
from src.case.aicam.aicam_template import AICamETLTemplate


class W_AI_CAM_LOG_STRANGER_F(AICamETLTemplate):
    def __init__(self):
        self.shop_d = None
        self.now = datetime.now()
        self.process_start_date = datetime(self.now.year, self.now.month, self.now.day) - timedelta(days=1)
        self.process_end_date = self.process_start_date + timedelta(days=1)

    def delete_data(self):
        current_date = int(self.now.strftime("%Y%m%d"))
        self.target_cursor.execute("""DELETE FROM W_AI_CAM_LOG_STRANGER_F WHERE DATE_WID = {}""".format(current_date))

    def get_source_data(self):
        start_of_day = int(time.mktime(self.process_start_date.timetuple()))*1000
        end_of_day = int(time.mktime(self.process_end_date.timetuple()))*1000
        self.source_cursor.execute("""SELECT t.id, t.detected_image_url, t.devicename, t.deviceID, t.placename, t.time_int, t.PersonID
                                FROM (SELECT *, CONVERT(time, INT) as time_int 
                                     FROM cctv.logs_stranger) as t
                                WHERE t.time_int < ? AND t.time_int > ? AND t.placeName  != 'HCM, 238 Nguyễn Oanh (Lầu 7, 8)'
                                ORDER BY t.time_int DESC
                                LIMIT 100""", (end_of_day, start_of_day))
        self.source_data = self.source_cursor.fetchall()
        columns = ['INTEGRATION_ID', 'DETECTED_IMAGE_URL', 'DEVICENAME', 'DEVICEID', 'SHOP_NM', 'TIME', 'PERSONID']
        self.source_data = pd.DataFrame(self.source_data)
        self.source_data.columns = columns
        self.source_data['TIME'] = pd.to_datetime(self.source_data['TIME'], unit='ms')

    def get_target_dim_data(self):
        self.target_cursor.execute("""SELECT SHOP_WID, SHOP_NM FROM W_SHOP_D 
                                    WHERE VALID_FM_DT <= TO_DATE('""" + self.process_start_date.strftime("%Y%m%d")+ """','YYYYMMDD')
                                    AND VALID_TO_DT > TO_DATE('""" + self.process_start_date.strftime("%Y%m%d")+ """','YYYYMMDD')""")
        self.shop_d = self.target_cursor.fetchall()
        self.shop_d = pd.DataFrame(self.shop_d)
        columns = ['SHOP_WID', 'SHOP_NM']
        self.shop_d.columns = columns

    def join_final_data(self):
        self.target_data = pd.merge(self.source_data, self.shop_d, on=['SHOP_NM'], how='left')
        self.target_data.drop('SHOP_NM', inplace=True, axis=1)
        current_date = int(self.now.strftime("%Y%m%d"))
        self.target_data['DATE_WID'] = current_date
        self.target_data['INTEGRATION_ID'] = self.target_data['INTEGRATION_ID'].apply(str)

    def insert_to_target(self):
        for row_count in range(0, self.target_data.shape[0]):
            chunk = self.target_data.iloc[row_count:row_count + 1, :].values.tolist()
            try:
                self.target_cursor.executemany("""INSERT INTO W_AI_CAM_LOG_STRANGER_F (
                                                INTEGRATION_ID,
                                                DETECTED_IMAGE_URL,
                                                DEVICENAME,
                                                DEVICEID,
                                                TIME,
                                                PERSONID,
                                                SHOP_WID,
                                                DATE_WID)
                                            VALUES (:1,:2,:3,:4,:5, :6, :7, :8)""", chunk)
            except Exception as err:
                print("Error W_AI_CAM_LOG_STRANGER_F : ORACLE : '{}' ".format(err))
        print("Đồng bộ thành công : W_AI_CAM_LOG_STRANGER_F")
