from abc import ABCMeta, abstractmethod
from src.connection import config_dict
from src.connection import connection_factory


class AICamETLTemplate:
    __metaclass__ = ABCMeta
    source = None
    target = None
    source_cursor = None
    target_cursor = None
    sheet_name = None
    data_row = None
    col = None
    source_data = None
    target_data = None

    def ETL(self):
        self.get_source()
        self.get_target()
        self.get_source_cursor()
        self.get_target_cursor()
        self.delete_data()
        self.get_source_data()
        self.get_target_dim_data()
        self.join_final_data()
        self.insert_to_target()
        self.commit()
        self.close()

    def get_source(self):
        self.source = connection_factory('MariaDB', config_dict)

    def get_target(self):
        self.target = connection_factory('oracle_dwh', config_dict)

    def get_source_cursor(self):
        self.source_cursor = self.source.cursor()

    def get_target_cursor(self):
        self.target_cursor = self.target.cursor()

    @abstractmethod
    def delete_data(self):
        pass

    @abstractmethod
    def get_source_data(self):
        pass

    @abstractmethod
    def get_target_dim_data(self):
        pass

    @abstractmethod
    def join_final_data(self):
        pass

    @abstractmethod
    def insert_to_target(self):
        pass

    def commit(self):
        self.target.commit()

    def close(self):
        self.source.close()
        self.target.close()
