import pypyodbc
import cx_Oracle
import gspread
import mariadb
from abc import ABCMeta, abstractmethod
from oauth2client.service_account import ServiceAccountCredentials

###############################################################################
# Behaviours
###############################################################################


class ConnectBehavior:
    __metaclass__ = ABCMeta
    connection = None

    @abstractmethod
    def connect(self, config):
        pass


class Connect2SQLServer(ConnectBehavior):
    def connect(self, config):
        try:
            self.connection = pypyodbc.connect(
                'DRIVER={ODBC Driver 17 for SQL Server};SERVER=' + config['host'] + ';DATABASE=' + config[
                    'database'] + ';UID=' + config['username'] + ';PWD=' + config['password']
            )
            print("Connect successfully SQL SERVER")

        except Exception as err:
            print(f"Error: '{err}'")
        return self.connection

    def __str__(self):
        return "SQLServer"


class Connect2Oracle(ConnectBehavior):
    def connect(self, config):
        try:
            dsn_tns = cx_Oracle.makedsn(config['host'], config['port'], service_name=config['servicename'])
            self.connection = cx_Oracle.connect(user=config['username'], password=config['password'], dsn=dsn_tns)
            print('Connect successfully ORACLE')
        except Exception as err:
            print(f"Error : '{err}'")
        return self.connection

    def __str__(self):
        return "Oracle"


class Connect2GGSheet(ConnectBehavior):
    def connect(self, config):
        try:
            credentials = ServiceAccountCredentials.from_json_keyfile_name(
                r'./connection/aws/F88 BI XLN-b68e11d2fd43.json', config['scope'])
            gc = gspread.authorize(credentials)
            # Lấy ID của worksheet
            spreadsheet_key = config['spreadsheet_key']
            self.connection = gc.open_by_key(spreadsheet_key)
            print('Connect successfully Google Sheet')
            return self.connection

        except Exception as err:
            print(f"Error : '{err}'")

    def __str__(self):
        return "GGSheet"


class Connect2MariaDB(ConnectBehavior):
    def connect(self, config):
        try:
            self.connection = mariadb.connect(
                user=config['username'],
                password=config['password'],
                host=config['host'],
                port=config['port'],

            )
            print('Connect successfully MariaDB')
        except Exception as err:
            print(f"Error : '{err}'")
        return self.connection

    def __str__(self):
        return "MariaDB"
